/*
 * Copyright (c) 2009, Stefan Krah
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>


#include "decDouble.h"


/* Nonsense version of escape-time algorithm for calculating a mandelbrot
 * set. Just for benchmarking. */
void
color_point(decDouble *x0, decDouble *y0, int maxiter, decContext *ctx)
{
	decDouble x, y, sq_x, sq_y, two, four;
	int i;
	
  	decDoubleFromString(&two, "2", ctx);
  	decDoubleFromString(&four, "4", ctx);

	decDoubleCopy(&x, x0);
	decDoubleCopy(&y, y0);
	decDoubleMultiply(&sq_x, &x, &x, ctx);
	decDoubleMultiply(&sq_y, &y, &y, ctx);

	for (i = 0; i < maxiter; i++) {
		decDoubleMultiply(&y, &x, &y, ctx);
		decDoubleMultiply(&y, &y, &two, ctx);
		decDoubleAdd(&y, &y, y0, ctx);
		decDoubleSubtract(&x, &sq_x, &sq_y, ctx);
		decDoubleAdd(&x, &x, x0, ctx);
		decDoubleMultiply(&sq_x, &x, &x, ctx);
		decDoubleMultiply(&sq_y, &y, &y, ctx);
	}

	decDoubleCopy(x0, &x);
}


int
main(int argc, char **argv)
{
	decContext ctx;
	decDouble x0, y0;
	long iter = 10000000;
	clock_t start_clock, end_clock;
	char s[DECQUAD_String];

	if (argc != 2) {
		fprintf(stderr, "usage: bench iter\n");
		exit(1);
	}
	iter = strtol(argv[1], NULL, 10);

	decContextDefault(&ctx, DEC_INIT_DECQUAD);

	decDoubleFromString(&x0, "0.222", &ctx);
	decDoubleFromString(&y0, "0.333", &ctx);

	start_clock = clock();
	color_point(&x0, &y0, iter, &ctx);
	end_clock = clock();

	printf("time: %f\n\n", (double)(end_clock-start_clock)/(double)CLOCKS_PER_SEC);

	decDoubleToString(&x0, s);
	printf("%s\n", s);

	return 0;
}


