/*
 * Copyright (c) 2009, Stefan Krah
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>


#include "decQuad.h"


/* Nonsense version of escape-time algorithm for calculating a mandelbrot
 * set. Just for benchmarking. */
void
color_point(decQuad *x0, decQuad *y0, int maxiter, decContext *ctx)
{
	decQuad x, y, x2, y2, two, four;
	int i = 0;
	
  	decQuadFromString(&two, "2", ctx);
  	decQuadFromString(&four, "4", ctx);

	decQuadCopy(&x, x0);
	decQuadCopy(&y, y0);
	decQuadMultiply(&x2, &x, &x, ctx);
	decQuadMultiply(&y2, &y, &y, ctx);

	for (i = 0; i < maxiter; i++) {
		decQuadMultiply(&y, &x, &y, ctx);
		decQuadMultiply(&y, &y, &two, ctx);
		decQuadAdd(&y, &y, y0, ctx);
		decQuadSubtract(&x, &x2, &y2, ctx);
		decQuadAdd(&x, &x, x0, ctx);
		decQuadMultiply(&x2, &x, &x, ctx);
		decQuadMultiply(&y2, &y, &y, ctx);
	}

	decQuadCopy(x0, &x);
}


int
main(int argc, char **argv)
{
	decContext ctx;
	decQuad x0, y0;
	long iter = 10000000;
	clock_t start_clock, end_clock;
	char s[DECQUAD_String];

	if (argc != 2) {
		fprintf(stderr, "usage: bench iter\n");
		exit(1);
	}
	iter = strtol(argv[1], NULL, 10);

	decContextDefault(&ctx, DEC_INIT_DECQUAD);

	decQuadFromString(&x0, "0.222", &ctx);
	decQuadFromString(&y0, "0.333", &ctx);

	start_clock = clock();
	color_point(&x0, &y0, iter, &ctx);
	end_clock = clock();

	printf("time: %f\n\n", (double)(end_clock-start_clock)/(double)CLOCKS_PER_SEC);

	decQuadToString(&x0, s);
	printf("%s\n", s);

	return 0;
}


